#ifndef PARSE_COMMON_H
#define PARSE_COMMON_H

#define YY_DECL int yylex \
 (YYSTYPE * yylval_param,YYLTYPE * yylloc_param ,yyscan_t yyscanner, parser_t *parser)


#include <stdio.h>
#include "kv.h"

typedef struct parser_s  parser_t;
typedef struct transation_s  transaction_t;

#include "fd.h"

#include "gram_execute.h"


#include "plumb_gram.tab.h"
#ifndef IN_LEX
#include "plumb_lex.h"
#endif


struct transation_s {
	exe_t *head, *tail;
	void *vars;
	int refcnt;
	char *name;               /* name of the transaction (what started it) - for diagnostics */
	fd_t *query_out;          /* where to write the output of query commands */
	char *err_on_flush;       /* if not NULL: special transaction, may not "flush;" */
};


struct parser_s {
	const char *templ;
	const char *inp_name;
	void *scanner;
	transaction_t *tx;
	yypstate *state;          /* for push-parsing */
	char *stream_leftover;    /* storing partial tokens between stream parse calls */
};


/* portable strdup() with support for NULL input */
char *strclone(const char *s);

/* allocate new string and concat s1 and s2 in it, free them and return the new
   string */
char *str_aconcat(char *s1, char *s2);

/* malloc() and generate a new anonymous ID */
char *anon_id(void);

/* keyeq for string hashes */
int strkeyeq(char *a, char *b);

/* remove leading and trailing single qchar and replace all known escape sequences */
char *str_quote(const char *si, char qchar);


/* optional prefixing for templates: if prefix is NULL, return id, else allocate
   new string and return prefix.id and free id */
char *template_prefix(const char *prefix, char *id);


int yy_var_subst(transaction_t *tx, char *name, int prot, void *scanner);

/* include fn from lex; free()s fn */
void yy_include(transaction_t *tx, const char *templ, char *fn);


/* parse a file in a new parser context, until eof; if tx is NULL, query_out is used for writing query results */
void plumb_parse_file(transaction_t *tx, const char *templ, const char *inp_name, FILE *in, fd_t *query_out);

/* parse a full script from a string in a new parser context, query_out is used for writing query results */
void plumb_parse_string(transaction_t *tx, const char *templ, const char *inp_name, const char *str, fd_t *query_out);

/* parse a stream, query_out is used for writing query results ; inp_name must be strclone()'d */
parser_t *plumb_parse_stream_open(transaction_t *tx, const char *templ, const char *inp_name, fd_t *query_out);
void plumb_parse_stream_buffer(parser_t *parser, char *s, int len);
void plumb_parse_stream_close(parser_t *parser);


void yyerror(void *yylloc, void *scanner, const char *s);
int yyparse ();
char *lex_getpos(void *scanner);
int lex_is_token_singlechar(int token);

/* realloc an array, clear all new items */
#define reCalloc(array, item_size, old_no_items, new_no_items) \
	do { \
		array = realloc(array, new_no_items * item_size); \
		memset((char *)(array+(old_no_items * item_size)), 0, (new_no_items - old_no_items) * item_size); \
	} while(0)

#endif
