/*  plumb - the pipe shell - echo string
    Copyright (C) 2012-2013  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page: http://repo.hu/projects/plumb
    Contact: plumb (at) igor2.repo.hu
*/

#include <stdlib.h>
#include <string.h>
#include "virt.h"
#include "_default.h"
#include "timer.h"

#include <stdio.h>
typedef struct {
	int len;
	char msg[1];
} echo_t;

/* should not write from start because other VPs are not yet started;
   set up a timer that tries to write. If that fails, we got blocked before the
   timer called. Also set up source block hook and write if source gets
   unblocked. Any write should end in a VC_QUIT to avoid multiple writes.
*/

static virt_ctrl_t echo_write(proc_virt_t *p)
{
	echo_t *e = p->data;

	vp_write(p, vstdout, e->msg, e->len);
	return VC_QUIT;
}

static virt_ctrl_t echo_timer(proc_virt_t *proc, void *data)
{
	if (!vp_is_blocked(proc, vstdout))
		return echo_write(proc);
	/* if we are blocked, we will write from echo_source_block */
	return VC_OK;
}

virt_ctrl_t echo_source_block(proc_virt_t *proc, int local_fd, int is_blocking)
{
	if ((local_fd == 1) && (!is_blocking))
		return echo_write(proc);

	return VC_OK;
}

static virt_ctrl_t echo_start(proc_virt_t *p)
{
	timer_add_delta(p, 0, NULL);
	return VC_OK;
}

static int echo_free(proc_virt_t *p)
{
	echo_t *e = p->data;
	free(e);
	p->data = NULL;
}

int vp_echo_init(proc_virt_t *p, kvlist_t *args)
{
	echo_t *e;
	int len, nonl;
	char *msg;

	vp_stdfds_set(p, VP_REQ_UNBOUND, VP_REQ_QUIT_ON_UNBIND, VP_REQ_UNBOUND, VP_REQ_UNBOUND, VP_REQ_UNBOUND, VP_REQ_UNBOUND);

	/* allocate a process descriptor */
	p->input = vp_input_discard;
	p->start = echo_start;
	p->timer = echo_timer;
	p->free  = echo_free;
	p->source_block = echo_source_block;

	nonl = 0;
	if (args != NULL) {
		if (args->head->next != NULL)
			plumb_error("%s=[echo] doesn't take multiple arguments\n", p->h.id);
		/* special case: arg is a single string */
		if (args->head->value == NULL) {
			msg = args->head->key;
		} else if (strcmp(args->head->key, "bin") == 0) {
			msg = args->head->value;
			nonl=1;
		} else if (strcmp(args->head->key, "text") == 0)
			msg = args->head->value;
		else
			plumb_error("%s=[echo]: unknown argument %s\n", p->h.id, args->head->key);
	}
	else {
		msg = "\n";
		nonl = 1;
	}

	/* allocate the message (leave room for \n) and copy msg */
	len = strlen(msg);
	e = malloc(len + sizeof(echo_t) + 2);
		memcpy(e->msg, msg, len);

	if (!nonl) {
		/* create message by appending a newline */
		e->len = len + 1;
		e->msg[len] = '\n';
		e->msg[len+1] = '\0';
	}
	else {
		e->len = len;
		e->msg[len] = '\0';
	}

	p->data = e;

	return 0;
}
