/*  plumb - the pipe shell - split the stream in fixed length records
    Copyright (C) 2012-2013  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page: http://repo.hu/projects/plumb
    Contact: plumb (at) igor2.repo.hu
*/
#include <stdlib.h>
#include <string.h>
#include "process.h"
#include "fd.h"
#include "virt.h"

typedef struct {
	int len;
	int used;
	char buff[1];
} fixlen_t;

static virt_ctrl_t fixlen_input(proc_virt_t *proc, int local_fd, const char *data, int len)
{
	fixlen_t *a = proc->data;

	/* if we have leftover, try to fill it up to a->len */
	if (a->used > 0) {
		int need, cpy;
		need = a->len - a->used;
		cpy = len >= need ? need : len;
		memcpy(a->buff + a->used, data, cpy);

		/* move pointer and modify lengths by bytes copied */
		len -= cpy;
		data += cpy;
		a->used += cpy;

		/* check if we have a full record accumulated */
		if (a->used == a->len) {
			vp_write(proc, 1, a->buff, a->used);
			a->used = 0;
		}
	}

	/* by now, data starts at the next full record or empty - copy full records */
	while(len >= a->len) {
		vp_write(proc, 1, data, a->len);
		len -= a->len;
		data += a->len;
	}

	/* partial record at the end
	   (buffer is empty && len < a->len) || (len == 0) if we got here
	   save the leftover */
	if (len > 0) {
		memcpy(a->buff, data, len);
		a->used = len;
	}

	return VC_OK;
}

void fixlen_free(proc_virt_t *p)
{
	fixlen_t *a = p->data;
	free(a);
}

int vp_fixlen_init(proc_virt_t *p, kvlist_t *args)
{
	fixlen_t *a;
	int len;
	char *end;
	char *slen = NULL;
	vp_args_t vargs[] = {
		{"", &slen},
		{"len", &slen},
		{NULL, NULL}
	};

	vp_stdfds_set(p, VP_REQ_QUIT_ON_UNBIND, VP_REQ_QUIT_ON_UNBIND, VP_REQ_UNBOUND, VP_REQ_UNBOUND, VP_REQ_UNBOUND, VP_REQ_UNBOUND);

	vp_args(args, vargs);

	if (slen == NULL)
		plumb_error("fixlen %s without len\n", p->h.id);

	len = strtol(slen, &end, 0);
	if (*end != '\0')
		plumb_error("fixlen %s: invalid len (must be an integer)\n", p->h.id);

	a = malloc(sizeof(fixlen_t) + len);
	a->len = len;
	a->used = 0;

	p->input = fixlen_input;
	p->free  = fixlen_free;
	p->data  = a;

	return 0;
}
