/*  plumb - the pipe shell - helpers for virtual processes (VPs)
    Copyright (C) 2012-2013  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page: http://repo.hu/projects/plumb
    Contact: plumb (at) igor2.repo.hu
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "genht/htsp.h"
#include "genht/hash.h"
#include "process.h"
#include "parse_common.h"
#include "plumb.h"
#include "vp.h"


/* list of name/init for all virtual process types */
struct vp_s {
	char *name;
	vp_init_cb init;
};

#include "virt/_init.c"

htsp_t *vp_table = NULL;

void vp_init(void)
{
	struct vp_s *v;
	
	vp_table = htsp_alloc(strhash, strkeyeq);
	for(v = vps; v->name != NULL; v++)
		htsp_set(vp_table, v->name, v->init);
}

void vp_uninit(void)
{
	htsp_free(vp_table);
}

proc_virt_t *vp_alloc(transaction_t *tx, char *cmd, const char *id, kvlist_t *args)
{
	proc_virt_t *p;
	vp_init_cb init;

	if (cmd != NULL) {
		init = htsp_get(vp_table, cmd);
		if (init == NULL)
			return NULL;
	}

	p = calloc(sizeof(proc_virt_t), 1);
	p->h.virt        = 1;
	p->h.id          = strclone(id);
	p->h.tx_name     = strclone(tx->name);
	p->h.sticky      = 0;
	p->h.respawn     = 0;
	p->h.kill_on_eof = 0;
	p->init          = init;
	p->tx            = tx;
	p->cmd           = cmd;

	vp_default(p);

	if (cmd != NULL)
		init(p, args);

	p->tx = NULL;

	return p;
}

void vp_free(proc_virt_t *p)
{
	if (p->cmd != NULL)
		free(p->cmd);
	if (p->free != NULL)
		p->free(p);
}

void vp_act(proc_virt_t *proc, virt_ctrl_t ret)
{
	switch(ret) {
		case VC_OK:    break;
		case VC_PAUSE: proc_pause(proc, 1); break;
		case VC_QUIT:  vp_stop(proc, 0); break;
	}
}

void vp_stop(proc_virt_t *proc, int hard)
{
	int n;
	/* virtual process: close all fds and mark it stopped */
	if (proc->stop != NULL)
		proc->stop(proc);
	for(n = 0; n < proc->h.num_fds; n++)
		fd_close(&proc->h.fds[n]);
	if (proc->h.sticky)
		plumb_sticky_died();
	timer_del_proc(proc);
	proc->h.state = PS_STOPPED;
}
